Method: Benchmark::IPS#ips

Defined in:
lib/benchmark/ips.rb

#ips(*args) {|job| ... } ⇒ Report

Measure code in block, each code’s benchmarked result will display in iteration per second with standard deviation in given time.

Parameters:

  • time (Integer)

    Specify how long should benchmark your code in seconds.

  • warmup (Integer)

    Specify how long should Warmup time run in seconds.

Yields:

  • (job)

Returns:



33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
# File 'lib/benchmark/ips.rb', line 33

def ips(*args)
  if args[0].is_a?(Hash)
    time, warmup, quiet = args[0].values_at(:time, :warmup, :quiet)
  else
    time, warmup, quiet = args
  end

  sync, $stdout.sync = $stdout.sync, true

  job = Job.new

  job_opts = {}
  job_opts[:time] = time unless time.nil?
  job_opts[:warmup] = warmup unless warmup.nil?
  job_opts[:quiet] = quiet unless quiet.nil?

  job.config job_opts

  yield job

  job.load_held_results

  job.run

  if job.run_single? && job.all_results_have_been_run?
    job.clear_held_results
  else
    job.save_held_results
    puts '', 'Pausing here -- run Ruby again to measure the next benchmark...' if job.run_single?
  end

  $stdout.sync = sync
  job.run_comparison
  job.generate_json

  report = job.full_report

  if ENV['SHARE'] || ENV['SHARE_URL']
    require 'benchmark/ips/share'
    share = Share.new report, job
    share.share
  end

  report
end